﻿namespace HeuristicLab.Algorithms.PushGP.Generators {
  using System;
  using System.Threading;

  using HeuristicLab.Algorithms.PushGP.Data.Random;
  using HeuristicLab.Core;

  public class NameGenerator {
    public const int DefaultRandomNameLength = 10;

    private static readonly ThreadLocal<IRandom> rand = RandomFactory.GetRandom();

    private static readonly char[] chars =
      "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890".ToCharArray();

    private int index;

    public string GetNextString() {
      var length = this.index < chars.Length ? 1 : (int)Math.Log(this.index, chars.Length) + 1;
      var result = new char[length];
      var tmp = this.index;
      this.index++;

      for (var i = length; i > 0; i--) {
        var a = (int)Math.Pow(chars.Length, i - 1);
        var b = tmp / a;

        result[length - i] = chars[b];

        tmp -= a * b;
      }

      return new string(result);
    }

    public static string RandomName(int length = DefaultRandomNameLength) {
      var name = new char[length];

      for (var i = 0; i < length; i++) name[i] = chars[rand.Value.Next(chars.Length)];

      return new string(name);
    }
  }
}